#!/usr/bin/env python

# Rnmap console client v. 0.2.0
#
# Copyright (C) 2000-2002 Tuomo Makinen (tmakinen@pp.htv.fi)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Rnmap cli client v. 0.2.0

Copyright (C) 2000-2002 Tuomo Makinen
Redistributable under the terms of the GNU General Public License

Usage:
    -s address       Rnmap server address
    -p port          Server port (default 3418)
    -f file          Filename to save the scanning result (no stdout)
    -n cmd           Nmap command
    -u user          Username
    -c file          Text file containing Username, Password and Server:Port information
                     (Newline separated)
                     
    --plain          Use plaintext communications
    --help           Prints this message
    --mparseable     Use machine parseable log form
    --scriptkiddie   Use script kiddie log form
    --xml            Use xml log form

    --version        Show version of rnmap cli client

    example:
    # rnmap.py -s 192.168.1.78 -n "-sS -p 1-65535 192.168.1.10" --mparseable"""

from socket import *
import sys
import os
import string
import getopt
import signal
import getpass
import stat
import signal
import errno

sys.path[:0] = ['../lib']

from client import *
from sec import *
if Use_cipher == 1:
    import cPickle

Port = 3418
Mode = " -oN"
File = None
Server = None
Nmapcmd = ""
Crypt = 1
Use_file = None

try:
    opts, args = getopt.getopt(sys.argv[1:], 's:p:n:f:u:c:', ['help', 'version', 'mparseable', 'scriptkiddie', 'xml', 'plain'])
except:
    fatalerror("Try 'rnmap.py --help' for more information.\n")

for o, a in opts:
    if o == '--version':
        sys.stdout.write("Rnmap cli client v. 0.2.0\n")
        sys.exit()
    if o == '-s': Server = a
    if o == '-p':
        try:
            Port = string.atoi(a)
        except ValueError:
            fatalerror("Invalid port specification.\n")
    if o == '-n':
        Nmapcmd = string.replace(a, ":", "###")
    if o == '-f': File = a
    if o == '-u':
        Logid = a
    if o == '-c':
        Use_file = a
    if o == '--plain': Crypt = 0
    if o == '--mparseable': Mode = " -oM"
    elif o == '--scriptkiddie': Mode = " -oS"
    elif o == '--xml': Mode = " -oX"
    if o == '--help':
        print __doc__
        sys.exit()

def read_configuration():
    try:
        fp = open(Use_file).readlines()
    except IOError, e:
        fatalerror("Can't open " + Use_file + ": " + e[1] + "\n")
    else:
        if stat.S_IMODE(os.lstat(Use_file)[stat.ST_MODE]) & 077:
            fatalerror("File " + Use_file + " must have no more than -rwx------ (700) permissions.\n")
        else:
            c = len(fp) - 1
            while c >= 0:
                if fp[c] == '\n':
                    del fp[c]
                c = c - 1
            try:
                global Server, Port
                logid = fp[0][:-1]
                passwd = fp[1][:-1]
                pindex = string.rfind(fp[2], ":")
                Server = fp[2][:pindex]
                Port = string.atoi(fp[2][pindex + 1:-1])
            except:
                fatalerror("Error reading configuration file!\n")
    return logid, passwd

def check_cipheruse():
    if Use_cipher == 0:
        sys.stderr.write("Can't find cryptographic modules!\n")
    elif check_randomsrc() == 0:
        sys.stderr.write("Can't find random source!\n")
    else:
        return
    fatalerror("Use --plain command line option to force plaintext communications mode if you know what you are doing.\n")
    
def connection(packet):
    global Sockobj
    Sockobj = socket(AF_INET, SOCK_STREAM)
    signal.alarm(10)
    try:
        Sockobj.connect((Server, Port))
    except error, e:
        if e[0] == 4:
            e = (110, 'Connection timed out')
        fatalerror(e[1] + "\n")
    signal.alarm(0)
    Sockobj.send(packet)
    data = ''; read = ''
    while 1:
        try:
            read = Sockobj.recv(4096)
        except error, e:
            if e[0] == errno.EINTR:
                continue
        if not read:
            break
        data = data + read    
    Sockobj.close()
    return data
    
def ask_username():
    try:
        logid = raw_input("Username for " + Server + ": ")
    except KeyboardInterrupt:
        fatalerror("\n")
    return logid

def ask_password():
    try:
        passwd = getpass.getpass()
    except KeyboardInterrupt:
        fatalerror("\n")
    return passwd

def message():
    global Key
    if Crypt == 1:
        publickey = fetch_key(Server)
        if publickey == None:
            message = KEY_REQUEST + Logid
        elif publickey == -1:
            fatalerror("Corrupted server key")
        else:
            message, Key = generate_message(Logid, Passwd, publickey, Nmapcmd + Mode)
    else:
        message = "100:" + Logid + ":" + Passwd + ":" + Nmapcmd + Mode
        
    result = connection(message)
    
    if Crypt == 1 and publickey  == None and result[:3] != "405":
        spkey = get_pubkey(Passwd, result)
        if spkey != -1:
            try:
                publickey = cPickle.loads(spkey[4:])
            except:
                fatalerror("Invalid server key.\n")
            else:
                save_key(spkey[4:], Server)
                message, Key = generate_message(Logid, Passwd, publickey, Nmapcmd + Mode)
                result = connection(message)
        else:
            fatalerror("Invalid server key.\n")
                
    if Crypt == 1 and result[:3] not in ("401", "405", "407"):
        iv = result[-8:]
        try:
            result = decipher(result[:-8], Key, iv)
        except:
            result = None
    if result == None or result[:3] not in STATUSHEADERS:
        sys.stdout.write(PROTOCOL_ERROR + "\n")
    else:
        if result[:3] not in ("201", "203"):
            sys.stdout.write(result[4:] + "\n")
        else:
            if File == None:
                sys.stdout.write(result[4:])
            else:
                try:
                    fp = open(File, "w")
                    fp.write(result[4:])
                    fp.close()
                except IOError, e:
                    fatalerror("Can't write to disk: " + e[1] + "\n")

def signal_handler_term(signum, frame):
    if signum == signal.SIGINT:
        sys.stderr.write("Received SIGINT, interrupting scan...\n")
    elif signum == signal.SIGTERM:
        sys.stderr.write("Received SIGTERM, interrupting scan...\n")
    # --
    if Crypt == 1:
        iv = key_generator(8)
        message = encipher(INTERRUPT_SCAN, Key, iv)
        message = message + iv
    else:
        message = INTERRUPT_SCAN
    Sockobj.send(message)

def fatalerror(error_message):
    sys.stderr.write(error_message)
    sys.exit(1)

if Use_file != None:
    Logid, Passwd = read_configuration()
   
if Server == None:
    print __doc__
    sys.exit(1)

if Crypt == 1:
    check_cipheruse()

os.umask(0066)

if Use_file == None:
    try:
        if Logid:
            Passwd = ask_password()
    except NameError:
        Logid = ask_username()
        Passwd = ask_password()
        
# Set signal handlers
signal.signal(signal.SIGALRM, signal_handler_sigalrm)
signal.signal(signal.SIGTERM, signal_handler_term)
signal.signal(signal.SIGINT, signal_handler_term)

message()
